""" Python strict datatypes for functions and objects. Check if the datatypes of the input parameters match the datatypes you want.
(GitHub) Written by: badlock """

from .strict_class import StrictClass
from .strict_function import StrictFunction
from .strict_type import strict_type
from .exceptions import AssignError
